from unittest import TestCase
from src import anodict


class Person:
    name: str
    age: int


class Employee:
    profession: str
    salary: float


class TestDictToClass(TestCase):
    def test_simple(self):
        o = anodict.dict_to_class({
            "name": "bob",
            "age": 23
        }, Person)
        self.assertEquals(o.name, "bob")
        self.assertEquals(o.age, 23)

    def test_ignore_unannotated(self):
        o = anodict.dict_to_class({
            "name": "bob",
            "age": 23,
            "height": 165
        }, Person, ignore_unannotated=True)
        self.assertEquals(o.name, "bob")
        self.assertEquals(o.age, 23)
        self.assertFalse(hasattr(o, 'height'))

    def test_nullify(self):
        o = anodict.dict_to_class({
            "name": "bob",
        }, Person, nullify_missing=True)
        self.assertEquals(o.name, "bob")
        self.assertEquals(o.age, None)



from typing import TypeVar, Generic

_T = TypeVar('_T')


class _Constructor(Generic[_T]):
    @staticmethod
    def dict_to_class(_dict: dict, _class: _T, ignore_unannotated: bool = False,
                      nullify_missing: bool = False) -> _T:
        if not hasattr(_class, '__annotations__'):
            raise ValueError('Class must have annotated properties.')
        if not set(_dict.keys()).issubset(_class.__annotations__.keys()) and not ignore_unannotated:
            raise ValueError('Dict contains unannotated properties.')
        _dict = _dict.copy()
        # ignore unannotated properties
        for k in set(_dict.keys()).difference(_class.__annotations__.keys()):
            del _dict[k]
        # nullify missing properties
        if nullify_missing:
            for k in set(_class.__annotations__.keys()).difference(_dict.keys()):
                _dict[k] = None
        instance = _class()
        setattr(instance, '__dict__', _dict)
        return instance


dict_to_class = _Constructor.dict_to_class
